;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Toy Koopa Boss, based on mikeyk's code
;;
;; Description: Toy Koopa Boss shoots bullet buills.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	EXTRA_PROP_1 = $7FAB28

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	dcb "INIT"	
	PHY
	JSR SubHorzPos
	TYA
	STA $157C,x
	PLY
	LDA $1588,x             ; if on the ground, reset the turn counter
	ORA #$04
	STA $1588,x             ; if on the ground, reset the turn counter
	LDA #$02		; Number of random values to use - 1.
	STA $1504,x
	RTL                 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	dcb "MAIN"	                
	PHB	 ; \
	PHK	 ;  | main sprite function, just calls local subroutine
	PLB	 ;  |
	JSR SPRITE_CODE_START   ;  |
	PLB	 ;  |
	RTL	 ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	SPRITE_GRAVITY = $20
	 
X_SPEED             dcb $08,$F8,$30,$D0,$F0,$10
KILLED_X_SPEED      dcb $F0,$10

RETURN
	RTS
SPRITE_CODE_START
	JSR Graphics     ; graphics routine
	LDA $14C8,x             ; \ 
	CMP #$08                ;  | if status != 8, return
	BNE RETURN              ; /
	LDA $9D                 ; \ if sprites locked, return
	BNE RETURN              ; /

	JSR SUB_OFF_SCREEN_X0   ; handle off screen situation
	JSR Charge_Init_Chk
	LDA $160E,x
	AND #$01
	BNE SKIP1504
	LDA $163E,x
	BNE SKIP1504
	LDA $1504,x
	BNE NO_THROW
	JSR THROW
	BRA SKIP1504
NO_THROW
	LDA $14
	AND #$3F
	BNE SKIP1504
	DEC $1504,x
	JSR SubHorzPos		; Face Mario
	TYA
	STA $157C,x
SKIP1504
	LDA $1588,x             ; \ if sprite is in contact with an object...
	AND #$03                ;  |
	BEQ NO_OBJ_CONTACT      ;  |
	LDA $160E,x
	AND #$01
	BEQ No_Charge
	STZ $160E,x
	LDA #$40
	STA $163E,x
	LDA #$09                ; \ sound effect
	STA $1DFC 
	STZ $B6,x
	LDA #$E0
	STA $AA,x
	LDA #$04
	CLC
	ADC $157C,x
	TAY
	LDA X_SPEED,y
	STA $B6,x
	JSL $01802A             ; update position based on speed values
	BRA DONE_WITH_SPEED
No_Charge
	LDA $157C,x             ;  | flip the direction status
	EOR #$01                ;  |
	STA $157C,x             ; /

NO_OBJ_CONTACT
	LDA $163E,x
	BNE ON_GROUND
	LDA EXTRA_PROP_1,x      ; don't stay on ledges if bit is set
	AND #$02                ;
	BEQ FALLING             ;

	LDA $1588,x             ; run the subroutine if the sprite is in the air...
	ORA $151C,x             ; ...and not already turning
	BNE ON_GROUND           ;
	JSR SUB_CHANGE_DIR      ;
	LDA #$01                ; set that we're already turning
	STA $151C,x             ;

ON_GROUND
	LDA $1588,x             ; if on the ground, reset the turn counter
	AND #$04
	BEQ IN_AIR
	STZ $151C,x
	LDA $163E,x
	STZ $AA,x
	LDA $1504,x
	BEQ STOP_X
	LDA $163E,x
	BNE STOP_X

	BRA X_TIME
FALLING
	LDA $1588,x             ; if on the ground, reset the turn counter
	AND #$04
	BEQ IN_AIR
	LDA #$10                ; \  y speed = 10
	STA $AA,x               ; /
X_TIME
	LDY $157C,x             ; \ set x speed based on direction
	LDA $160E,x
	BEQ No_Fast
	INY
	INY
No_Fast
	LDA X_SPEED,y           ;  |
	STA $B6,x               ; /
	BRA IN_AIR
STOP_X
	STZ $B6,x

IN_AIR
	JSL $01802A             ; update position based on speed values
DONE_WITH_SPEED
	JSL $018032             ; interact with sprites
	JSL $01A7DC             ; check for mario/sprite contact (carry set = contact)
	BCC RETURN_24           ; return if no contact
	JSR SUB_VERT_POS        ; 
	LDA $0E                 ; \ if mario isn't above sprite, and there's vertical contact...
	CMP #$E6                ;  |     ... sprite wins
	BPL SPRITE_WINS         ; /
	LDA $7D                 ; \if mario speed is upward, return
	BMI RETURN_24           ; /

	LDA $160E,x
	AND #$01
	BNE BOUNCE_OFF
	LDA $1504,x
	BEQ NEXT
	LDA $163E,x
	BEQ SPRITE_WINS
BOUNCE_OFF
	JSL $01AA33             ; set mario speed, NOTE: remove call to not bounce off sprite
	JSL $01AB99             ; display contact graphic
	LDA #$02
	STA $1DF9
	RTS
NEXT
	JSL $01AA33             ; set mario speed, NOTE: remove call to not bounce off sprite
	JSL $01AB99             ; display contact graphic

	LDA #$40
	STA $163E,x
	LDA $169E,x
	ORA #$01
	STA $160E,x
	STZ $B6,x

	JSR FireEnd
	JSR SUB_STOMP_PTS       ; give mario points
	INC $1528,x
	LDA $1528,x
	CMP #$04
	BEQ SPIN_KILL
RETURN_24B
	RTS	 ;
SPIN_KILL
	LDA #$04                ; \ status = 4 (being killed by spin jump)
	STA $14C8,x             ; /   
	LDA #$1F                ; \ set spin jump animation timer
	STA $1540,x             ; /
	JSL $07FC3B             ; show star animation
	LDA #$08                ; \ play sound effect
	STA $1DF9               ; /
	RTS                     ; return

SPRITE_WINS
	LDA $1490               ; \ if mario star timer > 0, goto HAS_STAR 
	BNE RETURN_24            ; / NOTE: branch to RETURN_24 to disable star killing                  
	JSL $00F5B7             ; hurt mario
RETURN_24
	RTS	 ; final return

Charge_Init_Chk
	LDA $1504,x
	BEQ RETURN_25
	LDA $163E,x
	BEQ RETURN_25
;	LDA $160E,x
;	BEQ  RETURN_25
	JSR SubHorzPos		; Face Mario
	TYA
	STA $157C,x
RETURN_25
	RTS

;============;
;RNG by 1024
;============;

RAND: 	
	db $01,$03,$03,$06

RANDOM:
	PHX
	PHP
	SEP #$30
	PHA
	JSL $01ACF9
	PLX
	CPX #$FF
	BNE NORMALRT
	LDA $148B
	BRA ENDRANDOM
NORMALRT:
	INX
	LDA $148B
	STA $4202
	STX $4203
	NOP
	NOP
	NOP
	NOP
	LDA $4217	
ENDRANDOM:
	PLP
	PLX
	RTL
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;Bullet Bill throw routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

X_OFFSET            dcb $14,$FD
X_OFFSET2           dcb $00,$FF

THROW
	LDA $1504,x
	BNE EndSpawn
	LDA $1534,x
	BNE FireRate
	LDA $1594,x
	CMP #$04
	BEQ FireEnd
	CMP #$03
	BEQ WhoDeal

	LDA #$18
	STA $1534,x
	INC $1594,x

	LDA $186C,x
	BNE EndSpawn
	JSL $02A9DE
	BMI EndSpawn

	LDA #$09                ; \ sound effect
	STA $1DFC 
	LDA #$08
	STA $14C8,y
	LDA #$1C
	STA $009E,y

                    PHY
                    LDA $157C,x
                    TAY
                    LDA $E4,x               ; \ set x position for new sprite
                    CLC
                    ADC X_OFFSET,y
                    PLY
                    STA $00E4,y

                    PHY
                    LDA $157C,x
                    TAY
                    LDA $14E0,x             ;  |
                    ADC X_OFFSET2,y
                    PLY
                    STA $14E0,y             ; /

                    LDA $D8,x               ; \ set y position for new sprite
                    SEC                     ;  | (y position of generator - 1)
                    SBC #$19                ;  |
                    STA $00D8,y             ;  |
                    LDA $14D4,x             ;  |
                    SBC #$00                ;  |
                    STA $14D4,y             ; /
	PHX
	TYX
	JSL $07F7D2
	PLX
                    LDA $157C,x
                    STA $00C2,y
EndSpawn
	RTS
FireRate
	DEC $1534,x
	RTS
WhoDeal
	INC $1594,x
	LDA #$20
	STA $1534,x
	RTS
FireEnd
	PHX
	LDA #$03		; Number of random values to use - 1.
	JSL RANDOM
	TAX 			; Transfer to X to use as an index
	LDA RAND,x
	PLX
	STA $1504,x
	STZ $1534,x
	STZ $1594,x
	RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL HorzIncY
	INY 
HorzIncY:
	RTS 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B829 - vertical mario/sprite position check - shared
; Y = 1 if mario below sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;org $03B829 

SUB_VERT_POS        LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
	LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
	SEC	;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
	SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
	STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
	LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
	SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
	BPL LABEL12            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
	INY	;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
LABEL12             RTS	;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_CHANGE_DIR
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;org $019098

SUB_CHANGE_DIR
	LDA $B6,x  
	EOR #$FF   
	INC A      
	STA $B6,x  
	LDA $157C,x
	EOR #$01   
	STA $157C,x
LABEL41             RTS        
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_STOMP_PTS
	LDA #$28
	STA $1DFC               ; /   
	LDA #$04               ; \
	STA $1697
	JSL $02ACE5             ; give mario points
	STZ $1697
	RTS                     ; return;

;##########################################
;###						###
;### 		    GRAPHICS ROUTINE		###
;###						###
;##########################################

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Tiles Informations
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TILEMAP_Normal
	dcb $00,$02,$20,$22,$04,$06,$24,$26
	dcb $00,$02,$20,$22,$04,$06,$24,$26
HORIZ_DISP_Normal
	dcb $00,$10,$00,$10,$00,$10,$00,$10
	dcb $10,$00,$10,$00,$10,$00,$10,$00
VERT_DISP_Normal
	dcb $F0,$F0,$00,$00,$F0,$F0,$00,$00
	dcb $F0,$F0,$00,$00,$F0,$F0,$00,$00
PROPERTIES_Normal
	dcb $25,$25,$25,$25,$25,$25,$25,$25
	dcb $65,$65,$65,$65,$65,$65,$65,$65 ;xyppccct format

TILEMAP_Cannon
	dcb $4E,$08,$0A,$28,$2A,$4D,$00,$00
	dcb $4E,$0A,$08,$2A,$28,$4D,$00,$00
HORIZ_DISP_Cannon
	dcb $0B,$00,$10,$00,$10,$03,$00,$00
	dcb $03,$00,$10,$00,$10,$0B,$00,$00
VERT_DISP_Cannon
	dcb $E8,$F0,$F0,$00,$00,$E8,$00,$00
	dcb $E8,$F0,$F0,$00,$00,$E8,$00,$00
PROPERTIES_Cannon
	dcb $35,$15,$15,$15,$15,$35,$00,$00
	dcb $75,$55,$55,$55,$55,$75,$00,$00

TILEMAP_Shell
	dcb $42,$40,$52,$50,$44,$46,$54,$56,$40,$42,$50,$52,$48,$4A,$58,$5A
	dcb $40,$42,$50,$52,$44,$46,$54,$56,$42,$40,$52,$50,$48,$4A,$58,$5A
HORIZ_DISP_Shell
	dcb $00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10
	dcb $00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10,$00,$10
VERT_DISP_Shell
	dcb $F8,$F8,$00,$00,$F8,$F8,$00,$00,$F8,$F8,$00,$00,$F8,$F8,$00,$00
	dcb $F8,$F8,$00,$00,$F8,$F8,$00,$00,$F8,$F8,$00,$00,$F8,$F8,$00,$00
PROPERTIES_Shell
	dcb $55,$55,$55,$55,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15,$15
	dcb $15,$15,$15,$15,$15,$15,$15,$15,$55,$55,$55,$55,$15,$15,$15,$15 ;xyppccct format

TILEMAP_Stun
	dcb $48,$4A,$58,$5A
	dcb $4A,$48,$5A,$58
HORIZ_DISP_Stun
	dcb $00,$10,$00,$10
	dcb $00,$10,$00,$10
VERT_DISP_Stun
	dcb $F8,$F8,$00,$00
	dcb $F8,$F8,$00,$00
PROPERTIES_Stun
	dcb $15,$15,$15,$15
	dcb $55,$55,$55,$55 ;xyppccct format
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN ROUTINES
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Graphics

	JSR GET_DRAW_INFO	; after: Y = index to sprite tile map ($300)
				;      $00 = sprite x position relative to screen boarder 
				;      $01 = sprite y position relative to screen boarder  
	LDA $1504,x
	BNE NEXT_FRAME1
	JSR FRAME_CANNON
	BRA Graphics_End
NEXT_FRAME1
	LDA $163E,x
	BEQ NEXT_FRAME2
	JSR FRAME_STUN
	BRA Graphics_End
NEXT_FRAME2
	LDA $160E,x
	AND #$01
	BEQ NEXT_FRAME3
	JSR FRAME_CHARGE
	BRA Graphics_End
NEXT_FRAME3
	JSR FRAME_WALK
	BRA Graphics_End
Graphics_End
	RTS
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; JSR Frames
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FRAME_WALK

	LDA $14                 ;\ 
	LSR A                   ; |
	LSR A                   ; |
	LSR A                   ; |
	CLC	 ; |
	ADC $15E9               ; |
	AND #$01                ; |
	ASL A                   ; |
	ASL A  
	STA $1602,x

	LDA $1602,x
	STA $03                 ; | $03 = index to frame start (0 or 4) 

	LDA $157C,x             ; \ $02 = sprite direction
	STA $02                 ; /
	BNE NO_ADD_WALK
	LDA $03
	CLC
	ADC #$08
	STA $03
NO_ADD_WALK
	PHX	 ; push sprite index
	LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START_2_WALK
	PHX	 ; push current tile number
	TXA	 ; \ X = index to horizontal displacement
	ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT_WALK
	TAX	 ; \ 
	
	LDA $00                 ; \ tile x position = sprite x location ($00)
	CLC	 ;  |
	ADC HORIZ_DISP_Normal,x        ;  |
	STA $0300,y             ; /
	
	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	CLC	 ;  |
	ADC VERT_DISP_Normal,x         ;  |
	STA $0301,y             ; /
	
	LDA TILEMAP_Normal,x           ; \ store tile
	STA $0302,y             ; / 

	LDA PROPERTIES_Normal,x        ;  | get tile properties using sprite direction
	ORA $64                 ;  | put in level properties
	STA $0303,y             ; / store tile properties
	
	PLX	 ; \ pull, X = current tile of the frame we're drawing
	INY	 ;  | increase index to sprite tile map ($300)...
	INY	 ;  |    ...we wrote 1 16x16 tile...
	INY	 ;  |    ...sprite OAM is 8x8...
	INY	 ;  |    ...so increment 4 times
	DEX	 ;  | go to next tile of frame and loop
	BPL LOOP_START_2_WALK        ; / 

	PLX	 ; pull, X = sprite index
	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$03                ;  | A = number of tiles drawn - 1
	JSL $01B7B3             ; / don't draw if offscreen
	RTS	 ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FRAME_CANNON

	STZ $1602,x

	LDA $1602,x
	STA $03                 ; | $03 = index to frame start (0 or 4) 

	LDA $157C,x             ; \ $02 = sprite direction
	STA $02                 ; /
	BNE NO_ADD_CANNON
	LDA $03
	CLC
	ADC #$08
	STA $03
NO_ADD_CANNON
	PHX	 ; push sprite index
	LDX #$05                ; loop counter = (number of tiles per frame) - 1
LOOP_START_2_CANNON
	PHX	 ; push current tile number
	TXA	 ; \ X = index to horizontal displacement
	ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT_CANNON
	TAX	 ; \ 
	
	LDA $00                 ; \ tile x position = sprite x location ($00)
	CLC	 ;  |
	ADC HORIZ_DISP_Cannon,x        ;  |
	STA $0300,y             ; /
	
	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	CLC	 ;  |
	ADC VERT_DISP_Cannon,x         ;  |
	STA $0301,y             ; /
	
	LDA TILEMAP_Cannon,x           ; \ store tile
	STA $0302,y             ; / 

	LDA PROPERTIES_Cannon,x        ;  | get tile properties using sprite direction
	ORA $64                 ;  | put in level properties
	STA $0303,y             ; / store tile properties
	
	PLX	 ; \ pull, X = current tile of the frame we're drawing
	INY	 ;  | increase index to sprite tile map ($300)...
	INY	 ;  |    ...we wrote 1 16x16 tile...
	INY	 ;  |    ...sprite OAM is 8x8...
	INY	 ;  |    ...so increment 4 times
	DEX	 ;  | go to next tile of frame and loop
	BPL LOOP_START_2_CANNON        ; / 

	PLX	 ; pull, X = sprite index
	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$05                ;  | A = number of tiles drawn - 1
	JSL $01B7B3             ; / don't draw if offscreen
	RTS	 ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FRAME_CHARGE

	LDA $14                 ;\ 
	LSR A                   ; |
	LSR A
	CLC	 ; |
	ADC $15E9               ; |
	AND #$03                ; |
	ASL A                   ; |
	ASL A  
	STA $1602,x

	LDA $1602,x
	STA $03                 ; | $03 = index to frame start (0 or 4) 

	LDA $157C,x             ; \ $02 = sprite direction
	STA $02                 ; /
	BNE NO_ADD_CHARGE
	LDA $03
	CLC
	ADC #$0C
	STA $03
NO_ADD_CHARGE
	PHX	 ; push sprite index
	LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START_2_CHARGE
	PHX	 ; push current tile number
	TXA	 ; \ X = index to horizontal displacement
	ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT_CHARGE
	TAX	 ; \ 
	
	LDA $00                 ; \ tile x position = sprite x location ($00)
	CLC	 ;  |
	ADC HORIZ_DISP_Shell,x        ;  |
	STA $0300,y             ; /
	
	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	CLC	 ;  |
	ADC VERT_DISP_Shell,x         ;  |
	STA $0301,y             ; /
	
	LDA TILEMAP_Shell,x           ; \ store tile
	STA $0302,y             ; / 

	LDA PROPERTIES_Shell,x        ;  | get tile properties using sprite direction
	ORA $64                 ;  | put in level properties
	STA $0303,y             ; / store tile properties
	
	PLX	 ; \ pull, X = current tile of the frame we're drawing
	INY	 ;  | increase index to sprite tile map ($300)...
	INY	 ;  |    ...we wrote 1 16x16 tile...
	INY	 ;  |    ...sprite OAM is 8x8...
	INY	 ;  |    ...so increment 4 times
	DEX	 ;  | go to next tile of frame and loop
	BPL LOOP_START_2_CHARGE        ; / 

	PLX	 ; pull, X = sprite index
	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$03                ;  | A = number of tiles drawn - 1
	JSL $01B7B3             ; / don't draw if offscreen
	RTS	 ; return
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FRAME_STUN

	STZ $1602,x

	LDA $1602,x
	STA $03                 ; | $03 = index to frame start (0 or 4) 

	LDA $157C,x             ; \ $02 = sprite direction
	STA $02                 ; /
	BNE NO_ADD_STUN
	LDA $03
	CLC
	ADC #$04
	STA $03
NO_ADD_STUN
	PHX	 ; push sprite index
	LDX #$03                ; loop counter = (number of tiles per frame) - 1
LOOP_START_2_STUN
	PHX	 ; push current tile number
	TXA	 ; \ X = index to horizontal displacement
	ORA $03                 ; / get index of tile (index to first tile of frame + current tile number)
FACING_LEFT_STUN
	TAX	 ; \ 
	
	LDA $00                 ; \ tile x position = sprite x location ($00)
	CLC	 ;  |
	ADC HORIZ_DISP_Stun,x        ;  |
	STA $0300,y             ; /
	
	LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
	CLC	 ;  |
	ADC VERT_DISP_Stun,x         ;  |
	STA $0301,y             ; /
	
	LDA TILEMAP_Stun,x           ; \ store tile
	STA $0302,y             ; / 

	LDA PROPERTIES_Stun,x        ;  | get tile properties using sprite direction
	ORA $64                 ;  | put in level properties
	STA $0303,y             ; / store tile properties
	
	PLX	 ; \ pull, X = current tile of the frame we're drawing
	INY	 ;  | increase index to sprite tile map ($300)...
	INY	 ;  |    ...we wrote 1 16x16 tile...
	INY	 ;  |    ...sprite OAM is 8x8...
	INY	 ;  |    ...so increment 4 times
	DEX	 ;  | go to next tile of frame and loop
	BPL LOOP_START_2_STUN        ; / 

	PLX	 ; pull, X = sprite index
	LDY #$02                ; \ 02, because we didn't write to 460 yet
	LDA #$03                ;  | A = number of tiles drawn - 1
	JSL $01B7B3             ; / don't draw if offscreen
	RTS	 ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B760 - graphics routine helper - shared
; sets off screen flags and sets index to OAM
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;org $03B75C

TABLE1              dcb $0C,$1C
TABLE2              dcb $01,$02

GET_DRAW_INFO
	STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ ON_SCREEN_X         ;  |
	INC $15A0,x             ; /

ON_SCREEN_X
	LDA $14E0,x             ; \
	XBA	 ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC	 ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC	 ;  |
	ADC.W #$0040            ;  |
	CMP.W #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE INVALID             ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
	INY	 ; / 
ON_SCREEN_LOOP
	LDA $D8,x               ; \ 
	CLC	 ;  | set vertical offscreen if necessary
	ADC TABLE1,y            ;  |
	PHP	 ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP	 ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ ON_SCREEN_Y         ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA TABLE2,y            ;  |
	STA $186C,x             ;  |
ON_SCREEN_Y
	DEY	 ;  |
	BPL ON_SCREEN_LOOP      ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC	 ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC	 ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

INVALID
	PLA	 ; \ return from *main gfx routine* subroutine...
	PLA	 ;  |    ...(not just this subroutine)
	RTS	 ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;org $03B83B             

TABLE3              dcb $40,$B0
TABLE6              dcb $01,$FF 
TABLE4              dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5              dcb $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0
	LDA #$06                ; \ entry point of routine determines value of $03
	BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1
	LDA #$04                ;  |
	BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2
	LDA #$02                ;  |
STORE_03
	STA $03                 ;  |
	BRA START_SUB           ;  |
SUB_OFF_SCREEN_X3
	STZ $03                 ; /
START_SUB
	JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
	BEQ RETURN_2            ; /    
	LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
	AND #$01                ;  |
	BNE VERTICAL_LEVEL      ; /     
	LDA $D8,x               ; \
	CLC	 ;  | 
	ADC #$50                ;  | if the sprite has gone off the bottom of the level...
	LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
	ADC #$00                ;  | 
	CMP #$02                ;  | 
	BPL ERASE_SPRITE        ; /    ...erase the sprite
	LDA $167A,x             ; \ if "process offscreen" flag is set, return
	AND #$04                ;  |
	BNE RETURN_2            ; /
	LDA $13                 ; \ 
	AND #$01                ;  | 
	ORA $03                 ;  | 
	STA $01                 ;  |
	TAY	 ; /
	LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
	CLC	 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
	ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
	ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
	CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
	PHP	 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
	LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
	LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
	ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
	PLP	 ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
	SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
	STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
	LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
	BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
	EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
	STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20
	LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
	BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE
	LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
	CMP #$08                ;  |
	BCC KILL_SPRITE         ; /
	LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
	CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
	BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
	LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
	STA $1938,y             ; /	         A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE
	STZ $14C8,x             ; erase sprite
RETURN_2
	RTS	 ; return
VERTICAL_LEVEL
	LDA $167A,x             ; \ if "process offscreen" flag is set, return
	AND #$04                ;  |
	BNE RETURN_2            ; /
	LDA $13                 ; \ only handle every other frame??
	LSR A                   ;  | 
	BCS RETURN_2            ; /
	AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
	STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
	TAY	 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
	LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
	CLC	 ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
	ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
	ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
	CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
	PHP	 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
	LDA.W $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
	LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
	ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
	PLP	 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
	SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
	STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
	LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
	BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
	EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
	STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22
	LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
	BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
	BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN
	LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
	ORA $186C,x             ;  |  
	RTS	 ; / return